#include <stdio.h>
#include <conio.h>
#include "forms3.h"

#define FORM_STUDENT 0
#define FORM_ADDRESS 1
#define BUT_RIGHT_ARROW 1
#define BUT_LEFT_ARROW  1
#define BUT_OK          2
#define COL_LBL   2
#define COL_INP0 17
#define COL_INP1 13

FormElement *element;

// Initializes the form elements. This procedure is called from initForm.
void initFormElements(uint8_t formIdx) {
    uint8_t row;

    switch (formIdx) {
        case FORM_STUDENT:
            row = 6;
            addField(ftNumNatural, 5, "Student ID", COL_LBL, row, COL_INP0, row, true);    // Field 1

            row += 2;
            newAndLinkWithPrevElement();
            addField(ftText, 20, "Name", COL_LBL, row, COL_INP0, row, true);               // Field 2

            row += 2;
            newAndLinkWithPrevElement();
            addField(ftDate, 10, "Date of birth", COL_LBL, row, COL_INP0, row, false);     // Field 3

            newAndLinkWithPrevElement();
            row += 2;
            addField(ftNumNatural, 8, "Course code", COL_LBL, row, COL_INP0, row, false);  // Field 4

            row += 2;
            newAndLinkWithPrevElement();
            addField(ftText, 1, "Gender (M/F)", COL_LBL, row, COL_INP0, row, false);       // Field 5

            row += 2;
            newAndLinkWithPrevElement();
            addField(ftFileName, 14, "Picture", COL_LBL, row, COL_INP0, row, false);       // Field 6

            row += 4;
            newAndLinkWithPrevElement();
            addButton(other, 1, row, 15, "-->");      // Button 1

            newAndLinkWithPrevElement();
            addButton(cancel, 20, row, 15, "Cancel"); // Button 2

            break;
        case FORM_ADDRESS:
            row = 6;
            addField(ftText, 30, "Street", COL_LBL, row, COL_INP1, row, false);         // Field 1

            row += 2;
            newAndLinkWithPrevElement();
            addField(ftNumNatural, 5, "Postcode", COL_LBL, row, COL_INP1, row, false);  // Field 2

            newAndLinkWithPrevElement();
            addField(ftText, 20, "City", COL_LBL + 18, row, COL_INP1 + 13, row, false); // Field 3

            row += 2;
            newAndLinkWithPrevElement();
            addField(ftText, 20, "Telephone", COL_LBL, row, COL_INP1, row, false);      // Field 4

            row += 2;
            newAndLinkWithPrevElement();
            addField(ftText, 30, "EMail", COL_LBL, row, COL_INP1, row, false);          // Field 5

            row += 4;
            newAndLinkWithPrevElement();
            addButton(other, 1, row, 15, "<--");     // Button 1

            newAndLinkWithPrevElement();
            addButton(ok, 20, row, 15, "OK");        // Button 2

            newAndLinkWithPrevElement();
            addButton(cancel, 39, row, 15, "Cancel"); // Button 3

            break;
    }
}


// Writes the current field's text and moves to the next element.
void nextField() {
  printf("%s\n", element->field->txt);
  element = element->next;
}


// Displays a headline.
void headline() {
  clearScreen();
  puts("FORMS3 Test\n\n\n");
}


// The main function.
int main(void) {
  bool processing;
  uint8_t currentFormIdx;
  Button *button;
  bool confirmed;
  
  
  setDecimalChar('.');
  setMessageLine(31);

  initForms(2, true);
  processing = true;
  currentFormIdx = 0;
  confirmed = false;

  do {
      headline();
      if (currentFormIdx == FORM_STUDENT) {
          puts("*** Form 0 - Student ***");
      }
      else {
          puts("*** Form 1 - Address ***");
      }

      selectForm(currentFormIdx);
      processForm();

      // Evaluate button activation
      if (formCancelled()) {
          // Cancel button activated
          processing = false;
      }
      else {
          switch (currentFormIdx) {
          case FORM_STUDENT:
              // Right arrow button activated
              button = getButton(BUT_RIGHT_ARROW);
              if (button->activated) {
                  currentFormIdx = FORM_ADDRESS;
                  button->activated = false;
              }
              else {
                  puts("Right arrow button should be activated, but it is not...");
                  processing = false;
              }
              break;
          case FORM_ADDRESS:
              button = getButton(BUT_LEFT_ARROW);
              if (button->activated) {
                  // Left arrow button activated
                  currentFormIdx = FORM_STUDENT;
                  button->activated = false;
              }
              else {
                  button = getButton(BUT_OK);
                  if (button->activated) {
                      confirmed = true;
                      button->activated = false;
                  }
                  else {
                      puts("Ok button should be activated, but it is not...");
                  }
                  processing = false;
              }
              break;
          default:
              printf("Unknown form index: %d\n", currentFormIdx);
          }
      }
  } while(processing);

  // Show the results
  
  headline();
  
  selectForm(FORM_STUDENT);
  element = getFormFirstElement();
  
  if (confirmed) {
      puts("Input was confirmed.\n");
  }
  else {
      puts("Input was cancelled.\n");
  }

  puts("*** FORM STUDENT ***");
  printf("Student ID   : "); nextField();
  printf("Name         : "); nextField();
  printf("Date of birth: "); nextField();
  printf("Course code  : "); nextField();
  printf("Gender       : "); nextField();
  printf("Picture      : "); nextField();
  
  selectForm(FORM_ADDRESS);
  element = getFormFirstElement();
  
  puts("\n*** FORM ADDRESS ***");
  printf("Street       : "); nextField();
  printf("Postcode     : "); nextField();
  printf("City         : "); nextField();
  printf("Telephone    : "); nextField();
  printf("EMail        : "); nextField();

  destroyAllForms();
  closeForms();

  waitForKey();

  return 0;
}
